-------------------------------------------------------------------------------
--                L  O  C  A  L     V  A  R  I  A  B  L  E  S
-------------------------------------------------------------------------------

local addonName, L = ...;

local GUILD_ROSTER_MAX_COLUMNS   = 5;
local GUILD_ROSTER_MAX_STRINGS   = 4;
local GUILD_ROSTER_BAR_MAX       = 239;
local GUILD_ROSTER_BUTTON_OFFSET = 2;
local GUILD_ROSTER_BUTTON_HEIGHT = 20;

local MOBILE_BUSY_ICON = "|TInterface\\ChatFrame\\UI-ChatIcon-ArmoryChat-BusyMobile:14:14:0:0:16:16:0:16:0:16|t";
local MOBILE_AWAY_ICON = "|TInterface\\ChatFrame\\UI-ChatIcon-ArmoryChat-AwayMobile:14:14:0:0:16:16:0:16:0:16|t";

local currentGuildView = view;

local filteredList = {[0] = -1};

local classFlag =
{
  ["DRUID"]       = 0x0001,
  ["DEATHKNIGHT"] = 0x0002,
  ["DEMONHUNTER"] = 0x0004,
  ["HUNTER"]      = 0x0008,
  ["MAGE"]        = 0x0010,
  ["MONK"]        = 0x0020,
  ["PALADIN"]     = 0x0040,
  ["PRIEST"]      = 0x0080,
  ["ROGUE"]       = 0x0100,
  ["SHAMAN"]      = 0x0200,
  ["WARLOCK"]     = 0x0400,
  ["WARRIOR"]     = 0x0800
}

local roleFlags =
{
  ["ALL"]  =
    classFlag.DRUID+  classFlag.DEATHKNIGHT+classFlag.DEMONHUNTER+
    classFlag.HUNTER+ classFlag.MAGE+       classFlag.MONK+
    classFlag.PALADIN+classFlag.PRIEST+     classFlag.ROGUE+
    classFlag.SHAMAN+ classFlag.WARLOCK+    classFlag.WARRIOR,
  ["TANK"] =
    classFlag.DRUID+  classFlag.DEATHKNIGHT+classFlag.DEMONHUNTER+
    classFlag.MONK+   classFlag.PALADIN+    classFlag.WARRIOR,
  ["HEAL"] =
    classFlag.DRUID+  classFlag.MONK+       classFlag.PALADIN+
    classFlag.PRIEST+ classFlag.SHAMAN,
}

local rankFlag =
{
  0x0001, 0x0002, 0x0004, 0x0008, 0x0010, 0x0020, 0x0040, 0x0080,
  0x0100, 0x0200, 0x0400, 0x0800, 0x1000, 0x2000, 0x4000, 0x8000,
};

local rosterFilterBtn = nil;
local inviteAllBtn = nil;
local inviteAllMask = nil;
local rosterFilters = nil;
local filterValues =
{
  name     = "",
  class    = roleFlags.ALL,
  zone     = "",
  note     = "",
  ranks    = 0xFFFF,
  levelMin = 0,
  levelMax = 0,
};

local origRosterUpdate = nil;

-- Taken from GUILD_ROSTER_COLUMNS in 'Blizard_GuildRoster.lua', patch 6.0.3
local rosterViews =
{
  playerStatus = true,
  guildStatus  = true,
  pvp          = true,
  achievement  = true,
  tradeskill   = true,
  reputation   = true,
};

-------------------------------------------------------------------------------
--               G  L  O  B  A  L     V  A  R  I  A  B  L  E  S
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
--             L  O  C  A  L     D  E  F  I  N  I  T  I  O  N  S
-------------------------------------------------------------------------------

local _L = {};

local rosterFiltersShow;

-------------------------------------------------------------------------------
--                L  O  C  A  L     F  U  N  C  T  I  O  N  S
-------------------------------------------------------------------------------

--
-- Function to process events for the 'Invite All' button
--
local function inviteAllButtonOnEvent(self, event, ...)
  -- Hide/Show the 'Invite All' mask, if required
  if ((NigglesDBC.settings.UiGuildInviteAll) and
      (inviteAllMask ~= nil)) then
    inviteAllMask:SetShown((UnitAffectingCombat("player")) or
      (GetNumGuildApplicants() == 0));
  end

  return;
end

--
-- Function to adjust the macro for the 'Invite All' button, just before its
-- 'OnClick' handler is called
--
local function inviteAllButtonPreClick(button)
  -- Local Variables
  local macroText = "/stopmacro [combat]\n";
  local name;
  local numApplicants = 0;

  -- Check the player isn't in combat
  if (not InCombatLockdown()) then
    -- Create the macro for the button
    numApplicants = GetNumGuildApplicants();
    for idx = 1, numApplicants do
      name = GetGuildApplicantInfo(idx);
      if ((name ~= nil) and (#name > 1)) then
        if (#macroText+#name+10 < 1023) then
          macroText = macroText.."/ginvite "..name.."\n";
        else
          break;
        end
      end
    end

    -- Set the button's macro text
    button:SetAttribute("macroText", macroText);
  end

  return;
end

--
-- Function to return the correct checked status for menu items
--
local function rosterFilterClassesIsChecked(menu, menuArgs, index, value)
  return (bit.band(filterValues.classes, value) ~= 0);
end

--
-- Function called when a button is clicked in the 'Classes' menu of the
-- Guild Roster filters
--
local function rosterFiltersClassesOnClick(menu, menuArgs, index, value, isChecked)
  -- Local Variables
  local level = menu:GetLevel();

  -- Work out which menu has been clicked on
  if (level == 1) then
    filterValues.classes = bit.bxor(filterValues.classes,
      (value or filterValues.classes));
  elseif (level == 2) then
    filterValues.classes = bit.bor(filterValues.classes, value);
  end

  -- Filter the Guild Roster
  NigglesGuildRosterFilter();

  return false, 2;
end

--
-- Function called when the 'Class' filter is clicked.
--
local function rosterFiltersClassesShow(self)
  -- Toggle the Classes menu
  Niggles.menu:Toggle(_L.menuRosterFiltersClasses, self);

  return;
end

--
-- Function to create the frames for the Guild Roster filters
--
local function rosterFiltersCreate()
  -- Local Variables
  local sortedClasses = {};

  -- Create the Niggles guild roster filters
  rosterFilters = CreateFrame("FRAME", nil, GuildRosterFrame,
    "NigglesGuildRosterFiltersTemplate");
    rosterFilters:SetFrameLevel(GuildRosterContainer:GetFrameLevel()-1);
  rosterFilters:SetScript("OnEvent", _L.rosterFiltersOnEvent);
  rosterFilters:RegisterEvent("GUILD_ROSTER_UPDATE");

  -- Set the script handlers for some of the frame's children
  rosterFilters.classes:SetScript("OnClick", rosterFiltersClassesShow);
  rosterFilters.ranks:SetScript("OnClick", _L.rosterFiltersRanksShow);

  -- Hook into frames that change the view
  GuildRosterShowOfflineButton:HookScript("OnClick",
    NigglesGuildRosterFilter);

  -- Create a list of the classes sorted by their localized names
  for class, _ in pairs(LOCALIZED_CLASS_NAMES_MALE) do
    table.insert(sortedClasses, class);
  end
  table.sort(sortedClasses,
    function(first, second)
      return LOCALIZED_CLASS_NAMES_MALE[first] <
               LOCALIZED_CLASS_NAMES_MALE[second];
    end);

  -- Add the classes to the Classes menu
  local menuItems = _L.menuRosterFiltersClasses.items;
  for _, class in ipairs(sortedClasses) do
    tinsert(menuItems,
      {
        type  = "CheckButton",
        label = LOCALIZED_CLASS_NAMES_MALE[class],
        value = classFlag[class],
        color = string.format("|cFF%02x%02x%02x",
          (255*RAID_CLASS_COLORS[class].r),
          (255*RAID_CLASS_COLORS[class].g),
          (255*RAID_CLASS_COLORS[class].b));
      });
  end

  return;
end

--
-- Function to process events for the Guild Roster filters
--
_L.rosterFiltersOnEvent = function(self, event, ...)
  -- Process the event
  if (event == "GUILD_ROSTER_UPDATE") then
    -- Filter the Guild Roster
    NigglesGuildRosterFilter();
  end

  return;
end

--
-- Function to return the correct checked status for 'Ranks' menu items
--
local function rosterFiltersRanksIsChecked(menu, menuArgs, index, value)
  return (bit.band(filterValues.ranks, value) ~= 0);
end

--
-- Function called when a button is clicked in the 'Ranks' menu of the
-- Guild Roster filters
--
local function rosterFiltersRanksOnClick(menu, menuArgs, index, value, isChecked)
  -- Check if the 'Uncheck All' button has been clicked
  if (index <= 2) then
    filterValues.ranks = value;
  else
    -- Update the class filter value
    if (isChecked) then
      filterValues.ranks = bit.bor(filterValues.ranks, value);
    else
      filterValues.ranks = bit.band(filterValues.ranks, bit.bnot(value));
    end
  end

  -- Filter the Guild Roster
  NigglesGuildRosterFilter();

  return false, 1;
end

--
-- Function called when the 'Ranks' menu is shown.
--
-- NOTE: This function is used to create the menu to cater for ranks being
--       renamed while the player is logged on.
--
local function rosterFiltersRanksItems()
  -- Local Variables
  local menuItems =
  {
    {type = "Button", label = CHECK_ALL,   value = 0xFFFF},
    {type = "Button", label = UNCHECK_ALL, value = 0x0000},
  };
  local numRanks = GuildControlGetNumRanks();

  -- Add items for each of the guild's ranks
  for idx = 1, numRanks do
    tinsert(menuItems,
      {
        type  = "CheckButton",
        label = GuildControlGetRankName(idx);
        value = rankFlag[idx];
      });
  end

  return menuItems;
end

--
-- Function called when the 'Ranks' filter is clicked.
--
function _L.rosterFiltersRanksShow(self)
  -- Toggle the 'Ranks' menu
  Niggles.menu:Toggle(_L.menuRosterFiltersRanks, self);

  return;
end

--
-- Function called when the user selects a different guild roster view
--
local function rosterFiltersSetView(view)
  -- Ensure the new view is valid
  if ((view == nil) or (rosterViews[view] == nil)) then
    view = "playerView";
  end

  -- Save the new view
  currentGuildView = view;

  -- Check if the filtering is enabled
  if ((NigglesDBC.settings.UiGuildRosterFilter) and
      (rosterFilterBtn ~= nil)) then
    -- Check if the new view is 'tradeskill'
    if (currentGuildView == "tradeskill") then
      -- Hide the Niggles guild roster filters
      rosterFiltersShow(false);

      -- Disable the button for toggling the Niggles guild roster
      rosterFilterBtn:Disable();
    else
      -- Make sure the button for toggling the Niggles guild roster is enabled
      rosterFilterBtn:Enable();
    end
  end

  return;
end

--
-- Function to show/hide the Niggles roster filters
--
rosterFiltersShow = function(show)
  -- Clear any selection
  SetGuildRosterSelection(0);
  GuildFrame.selectedGuildMember = 0;
  GuildMemberDetailFrame:Hide();

  -- Work out if the Niggles guild roster filters should be shown
  if (show == nil) then
    if (rosterFilters == nil) then
      show = true;
    else
      show = not rosterFilters:IsShown();
    end
  end

  -- Create the Niggles guild roster filters, if required
  if ((rosterFilters == nil) and (show)) then
    rosterFiltersCreate();
  end

  -- Show/Hide the Niggles guild roster filters
  if (rosterFilters ~= nil) then
    if (show) then
      -- Set all the filters to their default values
      rosterFilters.name:SetText("");
      filterValues.classes = roleFlags.ALL;
      rosterFilters.zone:SetText("");
      rosterFilters.levelMin:SetText("");
      rosterFilters.levelMax:SetText("");
      filterValues.ranks = 0xFFFF;
      rosterFilters.note:SetText("");

      -- Adjust the position of the guild roster
      GuildRosterContainer:SetPoint("TOP", 0, -170);

      -- Initialise the filtered list
      NigglesGuildRosterFilter();
    else
      -- Adjust the position of the guild roster
      GuildRosterContainer:SetPoint("TOP", 0, -95);

      -- Update the guild roster
      filteredList[0] = -1;
      GuildRoster_Update();
    end
    rosterFilters:SetShown(show);
  end

  return;
end

-------------------------------------------------------------------------------
--                                M  E  N  U  S
-------------------------------------------------------------------------------

_L.menuRosterFiltersClasses =
{
  style    = "menu",
  position = "right",
  offset   = {x = -18, y = -8},
  autoHide = false,
  onCheck  = rosterFilterClassesIsChecked,
  onClick  = rosterFiltersClassesOnClick,
  items    =
  {
    {
      type = "SubMenu",
      label = "Check",
      autoHide = false,
      onClick = rosterFiltersClassesOnClick,
      items =
      {
        {type = "Button", label = ALL,          value = roleFlags.ALL},
        {type = "Button", label = L["Tanking"], value = roleFlags.TANK},
        {type = "Button", label = L["Healing"], value = roleFlags.HEAL},
      }
    },
    {type = "Button", label = UNCHECK_ALL},
  }
};

_L.menuRosterFiltersRanks =
{
  style    = "menu",
  position = "right",
  offset   = {x = -18, y = -8},
  autoHide = false,
  onCheck  = rosterFiltersRanksIsChecked,
  onClick  = rosterFiltersRanksOnClick,
  items    = rosterFiltersRanksItems,
};

-------------------------------------------------------------------------------
--               G  L  O  B  A  L     F  U  N  C  T  I  O  N  S
-------------------------------------------------------------------------------

--
-- Function for processing clicks on the Niggles guild roster toggle
--
function NigglesGuildRosterButtonOnClick(self, button)
  -- Show/Hide the Niggles Guild Roster filters
  if (currentGuildView ~= "tradeskill") then
    rosterFiltersShow();
    PlaySound(SOUNDKIT.IG_MAINMENU_OPTION_CHECKBOX_ON);
  end

  return;
end

--
-- Function to update the Niggles guild roster
--
-- NOTE: This is a modified version of 'GuildRoster_Update()' in
--       'Blizzard_GuildRoster.lua', patch 6.0.3.
--
function NigglesGuildRosterUpdate()
  local scrollFrame = GuildRosterContainer;
  local offset = HybridScrollFrame_GetOffset(scrollFrame);
  local buttons = scrollFrame.buttons;
  local numButtons = #buttons;
  local button, index, class;
  local totalMembers, onlineMembers, onlineAndMobileMembers = GetNumGuildMembers();
  local selectedGuildMember = GetGuildRosterSelection();

  if ( currentGuildView == "tradeskill" ) then
    GuildRoster_UpdateTradeSkills();
    return;
  end

  local guildName, guildRankName, guildRankIndex = GetGuildInfo("player");
  local maxRankIndex = GuildControlGetNumRanks() - 1;
  -- Get selected guild member info
  local fullName, rank, rankIndex, level, class, zone, note, officernote, online, isAway, classFileName, achievementPoints, achievementRank, isMobile = GetGuildRosterInfo(selectedGuildMember);
  GuildFrame.selectedName = fullName;
  -- If there's a selected guildmember
  if ( selectedGuildMember > 0 ) then
    -- Update the guild member details frame
    if ( isMobile ) then
      if (isAway == 2) then
        GuildMemberDetailName:SetText(MOBILE_BUSY_ICON..GuildFrame.selectedName);
      elseif (isAway == 1) then
        GuildMemberDetailName:SetText(MOBILE_AWAY_ICON..GuildFrame.selectedName);
      else
        GuildMemberDetailName:SetText(ChatFrame_GetMobileEmbeddedTexture(73/255, 177/255, 73/255)..GuildFrame.selectedName);
      end
    else
      GuildMemberDetailName:SetText(GuildFrame.selectedName);
    end
    if ( level and class) then
      GuildMemberDetailLevel:SetFormattedText(FRIENDS_LEVEL_TEMPLATE, level, class);
    else
      GuildMemberDetailLevel:SetText("");
    end
    local zoneText = zone;
    if(isMobile and not online) then zoneText = REMOTE_CHAT; end;
    GuildMemberDetailZoneText:SetText(zoneText);
    GuildMemberDetailRankText:SetText(rank);
    if ( online ) then
      GuildMemberDetailOnlineText:SetText(GUILD_ONLINE_LABEL);
    else
      GuildMemberDetailOnlineText:SetText(GuildRoster_GetLastOnline(selectedGuildMember));
    end
    -- Update public note
    if ( CanEditPublicNote() ) then
      PersonalNoteText:SetTextColor(1.0, 1.0, 1.0);
      if ( (not note) or (note == "") ) then
        note = GUILD_NOTE_EDITLABEL;
      end
    else
      PersonalNoteText:SetTextColor(0.65, 0.65, 0.65);
    end
    GuildMemberNoteBackground:EnableMouse(CanEditPublicNote());
    PersonalNoteText:SetText(note);

    -- Manage guild member related buttons
    -- check if you can promote and the member is at least 2 ranks below you, or if you can demote and the member is not in the last rank, as well as below your rank
    if ( ( CanGuildPromote() and rankIndex > guildRankIndex + 1 ) or ( CanGuildDemote() and rankIndex < maxRankIndex and rankIndex > guildRankIndex ) ) then
      GuildMemberDetailRankLabel:SetHeight(20);
      GuildMemberRankDropdown:Show();
      UIDropDownMenu_SetText(GuildMemberRankDropdown, rank);
    else
      GuildMemberDetailRankLabel:SetHeight(0);
      GuildMemberRankDropdown:Hide();
    end

    -- Update officer note
    if ( CanViewOfficerNote() ) then
      if ( CanEditOfficerNote() ) then
        if ( (not officernote) or (officernote == "") ) then
          officernote = GUILD_OFFICERNOTE_EDITLABEL;
        end
        OfficerNoteText:SetTextColor(1.0, 1.0, 1.0);
      else
        OfficerNoteText:SetTextColor(0.65, 0.65, 0.65);
      end
      GuildMemberOfficerNoteBackground:EnableMouse(CanEditOfficerNote());
      OfficerNoteText:SetText(officernote);

      -- Resize detail frame
      GuildMemberDetailOfficerNoteLabel:Show();
      GuildMemberOfficerNoteBackground:Show();
      GuildMemberDetailFrame:SetHeight(GUILD_DETAIL_OFFICER_HEIGHT + GuildMemberDetailName:GetHeight() + GuildMemberDetailRankLabel:GetHeight());
    else
      GuildMemberDetailOfficerNoteLabel:Hide();
      GuildMemberOfficerNoteBackground:Hide();
      GuildMemberDetailFrame:SetHeight(GUILD_DETAIL_NORM_HEIGHT + GuildMemberDetailName:GetHeight() + GuildMemberDetailRankLabel:GetHeight());
    end

    if ( CanGuildRemove() and ( rankIndex >= 1 ) and ( rankIndex > guildRankIndex ) ) then
      GuildMemberRemoveButton:Enable();
    else
      GuildMemberRemoveButton:Disable();
    end
    if ( (UnitFullName("player") == fullName) or (not online) or isMobile ) then
      GuildMemberGroupInviteButton:Disable();
    else
      GuildMemberGroupInviteButton:Enable();
    end

    GuildFrame.selectedName = GetGuildRosterInfo(GetGuildRosterSelection());
  else
    GuildMemberDetailFrame:Hide();
  end

  local maxAchievementsPoints = GetGuildRosterLargestAchievementPoints();
  local visibleMembers = onlineAndMobileMembers;
  if ( GetGuildRosterShowOffline() ) then
    visibleMembers = totalMembers;
  end
  -- Modified for Niggles
  if (filteredList[0] >= 0) then
    visibleMembers = filteredList[0];
  end

  for i = 1, numButtons do
    button = buttons[i];
    -- Modified for Niggles
    if (filteredList[0] >= 0) then
      if (offset+i <= visibleMembers) then
        index = filteredList[offset+i];
      else
        index = 0;
      end
    else
      index = offset+i;
    end

    local fullName, rank, rankIndex, level, class, zone, note, officernote, online, isAway, classFileName, achievementPoints, achievementRank, isMobile, canSoR, repStanding = GetGuildRosterInfo(index);

    local onlineOrMobile = online or isMobile;

    if ((fullName) and (offset+i <= visibleMembers)) then
      button.guildIndex = index;
      local displayedName = Ambiguate(fullName, "guild");
      if ( isMobile and not online ) then
        if (isAway == 2) then
          displayedName = MOBILE_BUSY_ICON..displayedName;
        elseif (isAway == 1) then
          displayedName = MOBILE_AWAY_ICON..displayedName;
        else
          displayedName = ChatFrame_GetMobileEmbeddedTexture(73/255, 177/255, 73/255)..displayedName;
        end
      end
      button.online = onlineOrMobile;
      if ( currentGuildView == "playerStatus" ) then
        GuildRosterButton_SetStringText(button.string1, level, onlineOrMobile)
        button.icon:SetTexCoord(unpack(CLASS_ICON_TCOORDS[classFileName]));
        GuildRosterButton_SetStringText(button.string2, displayedName, onlineOrMobile, classFileName)
        local zoneText = zone;
        if(isMobile and not online) then zoneText = REMOTE_CHAT; end;
        GuildRosterButton_SetStringText(button.string3, zoneText, onlineOrMobile)

      elseif ( currentGuildView == "guildStatus" ) then
        GuildRosterButton_SetStringText(button.string1, displayedName, onlineOrMobile, classFileName)
        GuildRosterButton_SetStringText(button.string2, rank, onlineOrMobile)
        GuildRosterButton_SetStringText(button.string3, note, onlineOrMobile)

        if ( onlineOrMobile ) then
          GuildRosterButton_SetStringText(button.string4, GUILD_ONLINE_LABEL, onlineOrMobile);
        else
          GuildRosterButton_SetStringText(button.string4, GuildRoster_GetLastOnline(index), onlineOrMobile);
        end
      elseif ( currentGuildView == "pve" ) then
        GuildRosterButton_SetStringText(button.string1, level, onlineOrMobile);
        button.icon:SetTexCoord(unpack(CLASS_ICON_TCOORDS[classFileName]));
        GuildRosterButton_SetStringText(button.string2, displayedName, onlineOrMobile, classFileName);
        GuildRosterButton_SetStringText(button.string3, nil, onlineOrMobile);
        GuildRosterButton_SetStringText(button.string4, nil, onlineOrMobile);
      elseif ( currentGuildView == "achievement" ) then
        GuildRosterButton_SetStringText(button.string1, level, onlineOrMobile);
        button.icon:SetTexCoord(unpack(CLASS_ICON_TCOORDS[classFileName]));
        GuildRosterButton_SetStringText(button.string2, displayedName, onlineOrMobile, classFileName);
        if ( achievementPoints >= 0 ) then
          GuildRosterButton_SetStringText(button.string3, achievementPoints, onlineOrMobile);
          if ( achievementPoints == 0 ) then
            button.barTexture:Hide();
          else
            button.barTexture:SetWidth(achievementPoints / maxAchievementsPoints * GUILD_ROSTER_BAR_MAX);
            button.barTexture:Show();
          end
        else
          GuildRosterButton_SetStringText(button.string3, NO_ROSTER_ACHIEVEMENT_POINTS, onlineOrMobile);
          button.barTexture:Hide();
        end
        GuildRosterButton_SetStringText(button.barLabel, "#"..achievementRank, onlineOrMobile);
      elseif ( currentGuildView == "reputation" ) then
        GuildRosterButton_SetStringText(button.string1, level, onlineOrMobile)
        button.icon:SetTexCoord(unpack(CLASS_ICON_TCOORDS[classFileName]));
        GuildRosterButton_SetStringText(button.string2, displayedName, onlineOrMobile, classFileName);
        GuildRosterButton_SetStringText(button.string3, GetText("FACTION_STANDING_LABEL"..repStanding), onlineOrMobile);
      end
      button:Show();
      if ( mod(index, 2) == 0 ) then
        button.stripe:SetTexCoord(0.36230469, 0.38183594, 0.95898438, 0.99804688);
      else
        button.stripe:SetTexCoord(0.51660156, 0.53613281, 0.88281250, 0.92187500);
      end
      if ( selectedGuildMember == index ) then
        button:LockHighlight();
      else
        button:UnlockHighlight();
      end
    else
      button:Hide();
    end
  end
  local totalHeight = visibleMembers * (GUILD_ROSTER_BUTTON_HEIGHT + GUILD_ROSTER_BUTTON_OFFSET);
  local displayedHeight = numButtons * (GUILD_ROSTER_BUTTON_HEIGHT + GUILD_ROSTER_BUTTON_OFFSET);
  HybridScrollFrame_Update(scrollFrame, totalHeight, displayedHeight);
end

--
-- Function to filter the Niggles guild roster
--
function NigglesGuildRosterFilter()
  -- Local Variables
  local _;
  local visibleMembers;
  local name;
  local rankIdx;
  local level;
  local class;
  local zone;
  local note;
  local classFileName;
  local numMatches = 0;

  -- Check if the roster should be filtered
  if (not rosterFilters:IsShown()) then
    filteredList[0] = -1;
    return;
  end

  -- Work out how many guild members should be visible in the roster
  if (GetGuildRosterShowOffline()) then
    visibleMembers, _ = GetNumGuildMembers();
  else
    _, _, visibleMembers = GetNumGuildMembers();
  end

  -- Get some of the edit box filters
  filterValues.name = rosterFilters.name:GetText():lower();
  filterValues.zone = rosterFilters.zone:GetText():lower();
  filterValues.note = rosterFilters.note:GetText():lower();

  -- Get the level range
  filterValues.levelMin = (tonumber(rosterFilters.levelMin:GetText()) or 0);
  filterValues.levelMax = (tonumber(rosterFilters.levelMax:GetText()) or 0);
  if (filterValues.levelMin > 0) and (filterValues.levelMax == 0) then
    filterValues.levelMax = filterValues.levelMin;
  elseif (filterValues.levelMin == 0) and (filterValues.levelMax > 0) then
    filterValues.levelMin = filterValues.levelMax;
  elseif (filterValues.levelMin > filterValues.levelMax) then
    filterValues.levelMin, filterValues.levelMax =
      filterValues.levelMax, filterValues.levelMin;
  end

  -- Filter the guild members
  for idx = 1, visibleMembers do
    -- Get the required info for the member
    name, _, rankIdx, level, _, zone, note, _, _, _, classFileName =
      GetGuildRosterInfo(idx);

    -- Check if the member matches the filter
    if (((filterValues.name == nil) or
         ((name or ""):lower():find(filterValues.name, 1, true) ~= nil)) and
        (bit.band(filterValues.classes, classFlag[classFileName]) ~= 0) and
        ((filterValues.levelMin == 0) or
        ((level >= filterValues.levelMin) and
         (level <= filterValues.levelMax))) and
        (bit.band(filterValues.ranks, rankFlag[rankIdx+1]) ~= 0) and
        ((filterValues.zone == nil) or
         ((zone or ""):lower():find(filterValues.zone) ~= nil)) and
        ((filterValues.note == nil) or
         ((note or ""):lower():find(filterValues.note) ~= nil))) then
      numMatches = numMatches+1;
      filteredList[numMatches] = idx;
    end
  end
  filteredList[0] = numMatches;

  -- Update the Guild roster
  GuildRoster_Update();

  return;
end

--
-- Function to enable/disable the Niggles Guild UI features
--
function NigglesGuildUIEnable()
  -- Local Variables
  local msgBtn =  GuildRecruitmentMessageButton;

  -- Check the guild roster has been loaded
  if (GuildRosterFrame ~= nil) then
    -- Check the character isn't in combat
    if (not InCombatLockdown()) then
      -- Create the button for toggling the Niggles guild roster, if required
      if ((NigglesDBC.settings.UiGuildRosterFilter) and
          (rosterFilterBtn == nil)) then
        rosterFilterBtn = CreateFrame("BUTTON", nil, GuildRosterFrame,
          "NigglesGuildRosterButtonTemplate");
        rosterFilterBtn.icon:SetTexture("Interface\\Common\\UI-Searchbox-Icon");
        rosterFilterBtn.icon:SetTexCoord(0, 1, 0, 1);
      end

      -- Create the 'Invite All' button, if required
      if ((NigglesDBC.settings.UiGuildInviteAll) and
          (inviteAllBtn == nil)) then
        inviteAllBtn = CreateFrame("BUTTON", nil, GuildInfoFrameApplicants,
          "SecureActionButtonTemplate,UIPanelButtonTemplate");
        inviteAllBtn:SetAllPoints(msgBtn);
        inviteAllBtn:SetFrameStrata(msgBtn:GetFrameStrata());
        inviteAllBtn:SetFrameLevel(msgBtn:GetFrameLevel()+1);
        inviteAllBtn:SetText(L["Invite All"]);
        inviteAllBtn:SetScript("PreClick", inviteAllButtonPreClick);
        inviteAllBtn:SetAttribute("type", "macro");
        inviteAllBtn:RegisterEvent("LF_GUILD_RECRUITS_UPDATED");
        inviteAllBtn:RegisterEvent("LF_GUILD_RECRUIT_LIST_CHANGED");
        inviteAllBtn:RegisterEvent("PLAYER_REGEN_DISABLED");
        inviteAllBtn:RegisterEvent("PLAYER_REGEN_ENABLED");
        inviteAllBtn:SetScript("OnEvent", inviteAllButtonOnEvent);
      end

      -- Create the mask for the 'Invite All' button, if required
      if ((NigglesDBC.settings.UiGuildInviteAll) and
          (inviteAllBtn ~= nil) and
          (inviteAllMask == nil)) then
        inviteAllMask = CreateFrame("BUTTON", nil, GuildInfoFrameApplicants,
          "NigglesDisabledButtonTemplate");
        inviteAllMask:SetAllPoints(inviteAllBtn);
        inviteAllMask:SetFrameStrata(inviteAllBtn:GetFrameStrata());
        inviteAllMask:SetFrameLevel(inviteAllBtn:GetFrameLevel()+1);
        inviteAllMask.text:SetText(L["Invite All"]);
      end

      -- Show/Hide the buttons
      if (rosterFilterBtn ~= nil) then
        rosterFilterBtn:SetShown(NigglesDBC.settings.UiGuildRosterFilter);
      end
      if (inviteAllBtn ~= nil) then
        inviteAllBtn:SetShown(NigglesDBC.settings.UiGuildInviteAll);
        msgBtn:SetShown(not NigglesDBC.settings.UiGuildInviteAll);
      end

      -- Replace/restore the function used to update the Guild roster
      if (origRosterUpdate == nil) then
        origRosterUpdate = GuildRoster_Update;
      end
      if (NigglesDBC.settings.UiGuildRosterFilter) then
        GuildRoster_Update = NigglesGuildRosterUpdate;
      else
        GuildRoster_Update = origRosterUpdate;
      end
      GuildRosterContainer.update = GuildRoster_Update;

      -- Get the current guild view
      rosterFiltersSetView(GetCVar("guildRosterView"));
    else
      tinsert(Niggles.postCombat, NigglesGuildUIEnable);
    end
  end

  return;
end

--
-- Function called when Blizzard's Guild Roster addon is loaded
--
function NigglesGuildUIOnLoad()
  -- Local Variables
  local button;
  local level;
  local prevButton = nil;

  -- Hook into the function that changes the view
  hooksecurefunc("GuildRoster_SetView", rosterFiltersSetView);

  -- Hook the 'Request' tab to enable/disable the 'Invite All' button
  GuildInfoFrameApplicants:HookScript("OnShow", inviteAllButtonOnEvent);

  -- Adjust the position of some of the Guild Roster frames
  level = GuildRosterContainer:GetFrameLevel()+1;
  GuildRosterContainer:ClearAllPoints();
  GuildRosterContainer:SetPoint("TOP", 0, -95);
  GuildRosterContainer:SetPoint("BOTTOM", 0, 30);
  GuildRosterContainer:SetPoint("RIGHT", -27, 0);
  GuildRosterContainer:SetFrameLevel(level);
  for idx = 1, 5 do
    button = _G["GuildRosterColumnButton"..idx];
    button:SetFrameLevel(level);
    button:ClearAllPoints();
    if (prevButton == nil) then
      button:SetPoint("BOTTOMLEFT", GuildRosterContainer, "TOPLEFT", -2, 3);
    else
      button:SetPoint("TOPLEFT", prevButton, "TOPRIGHT", -2, 0);
    end
    prevButton = button;
  end

  -- Enable/disable the Niggles guild features
  NigglesGuildUIEnable();

  return;
end
